@@ -2,7 +2,7 @@
 // detail/push_options.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2011 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2013 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -103,7 +103,9 @@
 # pragma warning (disable:4180)
 # pragma warning (disable:4244)
 # pragma warning (disable:4355)
+# pragma warning (disable:4510)
 # pragma warning (disable:4512)
+# pragma warning (disable:4610)
 # pragma warning (disable:4675)
 # if defined(_M_IX86) && defined(_Wp64)
 // The /Wp64 option is broken. If you want to check 64 bit portability, use a
@@ -124,4 +126,13 @@
 #  error Multithreaded RTL must be selected.
 # endif // !defined(_MT)
 
+# if defined(__cplusplus_cli) || defined(__cplusplus_winrt)
+#  if !defined(BOOST_ASIO_DISABLE_CLR_WORKAROUND)
+#   if !defined(generic)
+#    define generic cpp_generic
+#    define BOOST_ASIO_CLR_WORKAROUND
+#   endif
+#  endif
+# endif
+
 #endif
