@@ -8,7 +8,8 @@
 // See http://www.boost.org/libs/move for documentation.
 //
 //////////////////////////////////////////////////////////////////////////////
-#include <boost/move/move.hpp>
+#include <boost/move/detail/config_begin.hpp>
+#include <boost/move/utility.hpp>
 
 //[clone_ptr_base_derived
 class Base
@@ -152,3 +153,5 @@ int main()
    //]
    return 0;
 }
+
+#include <boost/move/detail/config_end.hpp>
